/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.
 */

package com.huawei.agc.flutter.appmessaging.utils;

import io.flutter.Log;

public interface ValueGetter {
    String TAG = ValueGetter.class.getName();

    /**
     * Converts a Object into a String.
     *
     * @param key:   String key.
     * @param value: Object value.
     * @return map
     */
    static String getString(String key, Object value) {
        if (!(value instanceof String)) {
            Log.w(TAG, "toString | String value expected for " + key + ". ");
            return "";
        }
        return (String) value;
    }

    /**
     * Converts a Object into a Boolean.
     *
     * @param key:   String key.
     * @param value: Object value.
     * @return map
     */
    static Boolean toBoolean(String key, Object value) {
        if (!(value instanceof Boolean)) {
            Log.w(TAG, "toBoolean | Boolean value expected for " + key + ". Returning false as default.");
            return false;
        }
        return (Boolean) value;
    }

    /**
     * Converts a Object into a Integer.
     *
     * @param key:   String key.
     * @param value: Object value.
     * @return map
     */
    static Integer toInteger(String key, Object value) {
        if (!(value instanceof Integer)) {
            Log.w(TAG, "toInteger | Integer value expected for " + key);
            return null;
        }
        return (Integer) value;
    }
}
