/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.
 */

package com.huawei.agc.flutter.appmessaging.viewmodel;

import android.app.Activity;

import com.huawei.agc.flutter.appmessaging.presenter.AGCAppMessagingContract;
import com.huawei.agc.flutter.appmessaging.presenter.AGCAppMessagingContract.ResultListener;
import com.huawei.agconnect.AGConnectInstance;
import com.huawei.agconnect.appmessaging.AGConnectAppMessaging;
import com.huawei.agconnect.appmessaging.Location;

public class AGCAppMessagingViewModel implements AGCAppMessagingContract.Presenter {
    private final Activity mActivity;
    private final AGConnectAppMessaging instance;

    public AGCAppMessagingViewModel(Activity activity, AGConnectAppMessaging agConnectAppMessaging) {
        this.mActivity = activity;
        initAGConnectInstance();
        this.instance = agConnectAppMessaging;
    }

    private void initAGConnectInstance() {
        if (AGConnectInstance.getInstance() == null) {
            AGConnectInstance.initialize(mActivity.getApplicationContext());
        }
    }

    @Override
    public void removeCustomView() {
        instance.removeCustomView();
    }

    @Override
    public void setDisplayLocation(int locationValue) {
        if (locationValue == 0) {
            instance.setDisplayLocation(Location.BOTTOM);
        } else {
            instance.setDisplayLocation(Location.CENTER);
        }
    }

    @Override
    public void trigger(String eventId) {
        instance.trigger(eventId);
    }

    @Override
    public void setFetchMessageEnable(Boolean enabled) {
        instance.setFetchMessageEnable(enabled);
    }

    @Override
    public void setDisplayEnable(Boolean enabled) {
        instance.setDisplayEnable(enabled);
    }

    @Override
    public void isDisplayEnable(ResultListener<Boolean> resultListener) {
        boolean enabled = instance.isDisplayEnable();
        resultListener.onSuccess(enabled);
    }

    @Override
    public void isFetchMessageEnable(ResultListener<Boolean> resultListener) {
        boolean enabled = instance.isFetchMessageEnable();
        resultListener.onSuccess(enabled);
    }

    @Override
    public void setForceFetch() {
        instance.setForceFetch();
    }

}
